/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 2; tab-width: 2 -*- */
/* test-node.c
 *
 * Copyright (C) 2001 Paolo Casarini <paolo@casarini.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */

#include <glib.h>
#include <stdio.h>
#include <parser.h>
#include <gdome.h>
#include <gdome-xml-str.h>

void
dot() {
	printf(".");
	fflush(stdout);
}

gboolean
test_node(GdomeNode *el) {
	GdomeNode *node1, *node2, *node3, *node4, *node5, *tnode, *cnode;
	GdomeNodeList *nl;
  GdomeNamedNodeMap *nnm;
	GdomeException exc;
  GdomeDOMString *str;
  gboolean ret = TRUE;
  int i;

	nl = el->vtab->childNodes(el, &exc);
  node1 = nl->vtab->item(nl, 1L, &exc);
  node2 = nl->vtab->item(nl, 3L, &exc);
	node3 = nl->vtab->item(nl, 5L, &exc);
	node4 = nl->vtab->item(nl, 7L, &exc);
  node5 = nl->vtab->item(nl, 9L, &exc);

  /*------------------- TEST ATTRIBUTES ------------------*/

	/* Test Node.attributes */
	nnm = node3->vtab->attributes(node3, &exc);
	if (nnm == NULL)
		fprintf(stderr,"\nNode.attributes, NULL"), ret = FALSE;
	else if(nnm->vtab->length(nnm, &exc) != 2)
		fprintf(stderr,"\nNode.attributes, wrong length"), ret = FALSE;
	dot();

	/* Test Node.firstChild */
  if (el->vtab->firstChild(el, &exc) != nl->vtab->item(nl, 0L, &exc))
		fprintf(stderr,"\nNode.firstChild, error"), ret = FALSE;
  dot();

	/* Test Node.lastChild */
  if (el->vtab->lastChild(el, &exc) != nl->vtab->item(nl, nl->vtab->length(nl, &exc) - 1, &exc))
		fprintf(stderr,"\nNode.lastChild, error"), ret = FALSE;
	dot();

	/* Test Node.localName */
	str = node2->vtab->localName(node2, &exc);
  if(!xmlStrEqual(str->str,"NODE2"))
		fprintf(stderr,"\nNode.localName, wrong on NODE2"), ret = FALSE;
  str->unref(str);
	str = node3->vtab->localName(node3, &exc);
  if(str != NULL)
		fprintf(stderr,"\nNode.localName, wrong on NODE3"), ret = FALSE;
  dot();

	/* Test Node.namespaceURI */
	str = node2->vtab->namespaceURI(node2, &exc);
  if(!xmlStrEqual(str->str,"urn:test.tst"))
		fprintf(stderr,"\nNode.namespaceURI, wrong on NODE2"), ret = FALSE;
  str->unref(str);
	str = node3->vtab->namespaceURI(node3, &exc);
  if(str != NULL)
		fprintf(stderr,"\nNode.namespaceURI, wrong on NODE3"), ret = FALSE;
	dot();

	/* Test Node.nextSibling */
  if (node2->vtab->nextSibling(node2, &exc) != nl->vtab->item(nl, 4L, &exc))
		fprintf(stderr,"\nNode.nextSibling, error"), ret = FALSE;
	dot();

	/* Test Node.nodeName */
	str = node1->vtab->nodeName(node1, &exc);
  if(!xmlStrEqual(str->str,"NODE1"))
		fprintf(stderr,"\nNode.nodeName, wrong on NODE1"), ret = FALSE;
  str->unref(str);
	str = node2->vtab->nodeName(node2, &exc);
	if(!xmlStrEqual(str->str,"tns:NODE2"))
		fprintf(stderr,"\nNode.nodeName, wrong on NODE2"), ret = FALSE;
	str->unref(str);
  tnode = node1->vtab->firstChild(node1, &exc);
	str = tnode->vtab->nodeName(tnode, &exc);
	if(!xmlStrEqual(str->str,"#text"))
		fprintf(stderr,"\nNode.nodeName, wrong on TNODE"), ret = FALSE;
	str->unref(str);
	tnode->vtab->unref(tnode, &exc);
  dot();

  /* Test Node.nodeType */
  if (node2->vtab->nodeType(node2, &exc) != XML_ELEMENT_NODE)
		fprintf(stderr,"\nNode.nextType, error"), ret = FALSE;
	dot();

  /* Test Node.nodeValue */
  tnode = node2->vtab->firstChild(node2, &exc);
	str = tnode->vtab->nodeValue(tnode, &exc);
	if(!xmlStrEqual(str->str,"This is a node with a namespace"))
		fprintf(stderr,"\nNode.nodeValue, wrong on TNODE"), ret = FALSE;
	str->unref(str);
	tnode->vtab->unref(tnode, &exc);
	dot();

  /* Test Node.ownerDocument */
	if (node2->vtab->ownerDocument(node2, &exc) !=
			node3->vtab->ownerDocument(node3, &exc))
		fprintf(stderr,"\nNode.ownerDocument, error"), ret = FALSE;
	dot();

  /* Test Node.parentNode */
  if (node2->vtab->parentNode(node2, &exc) != el)
		fprintf(stderr,"\nNode.parentNode, error"), ret = FALSE;
	dot();
  
	/* Test Node.prefix */
	str = node2->vtab->prefix(node2, &exc);
  if(!xmlStrEqual(str->str,"tns"))
		fprintf(stderr,"\nNode.prefix, wrong on NODE2"), ret = FALSE;
  str->unref(str);
	str = node3->vtab->prefix(node3, &exc);
  if(str != NULL)
		fprintf(stderr,"\nNode.prefix, wrong on NODE3"), ret = FALSE;
	dot();

	/* Test Node.previousSibling */
  if (node2->vtab->previousSibling(node2, &exc) != nl->vtab->item(nl, 2L, &exc))
		fprintf(stderr,"\nNode.previousSibling, error"), ret = FALSE;
	dot();

	/* Test Node::set_nodeValue */
	tnode = node2->vtab->firstChild(node2, &exc);
	str = gdome_xml_str_mkref ("xxxxx");
	tnode->vtab->set_nodeValue(tnode, str, &exc);
  str->unref(str);
	str = tnode->vtab->nodeValue(tnode, &exc);
	if(!xmlStrEqual(str->str,"xxxxx"))
		fprintf(stderr,"\nNode.set_nodeValue, error"), ret = FALSE;
	str->unref(str);
	tnode->vtab->unref(tnode, &exc);

	str = gdome_xml_str_mkref ("ATTR1");
	tnode = nnm->vtab->getNamedItem(nnm, str, &exc);
  str->unref(str);
	str = gdome_xml_str_mkref ("Ciao");
	tnode->vtab->set_nodeValue(tnode, str, &exc);
  str->unref(str);
	str = tnode->vtab->nodeValue(tnode, &exc);
	if(!xmlStrEqual(str->str,"Ciao"))
		fprintf(stderr,"\nNode.set_nodeValue, error"), ret = FALSE;
	str->unref(str);
	tnode->vtab->unref(tnode, &exc);

	dot();

	/* Test Node::set_prefix */
	str = gdome_xml_str_mkref ("xxx");
	node2->vtab->set_prefix(node2, str, &exc);
  str->unref(str);
	str = node2->vtab->prefix(node2, &exc);
	if(!xmlStrEqual(str->str,"xxx"))
		fprintf(stderr,"\nNode.set_prefix, error"), ret = FALSE;
	str->unref(str);
	dot();

  /*------------------- TEST METHODS ------------------*/

  /* Test Node::hasChildNodes */
  if (!node1->vtab->hasChildNodes(node1, &exc))
		fprintf(stderr,"\nNode.hasChildNodes(), error on NODE1"), ret = FALSE;
  if (node4->vtab->hasChildNodes(node4, &exc))
		fprintf(stderr,"\nNode.hasChildNodes(), error on NODE4"), ret = FALSE;
  dot();

  /* Test Node::hasAttributes */
  if (!node3->vtab->hasAttributes(node3, &exc))
		fprintf(stderr,"\nNode.hasAttributes(), error on NODE3"), ret = FALSE;
  if (node1->vtab->hasAttributes(node1, &exc))
		fprintf(stderr,"\nNode.hasAttributes(), error on NODE1"), ret = FALSE;
  dot();

  /* Test Methods... reverse the order of the nodes */
	el->vtab->appendChild(el, node1, &exc);
	node5 = el->vtab->replaceChild(el, node4, node5, &exc);
  el->vtab->insertBefore(el, node2, node1, &exc);
	el->vtab->insertBefore(el, node3, node2, &exc);
  node4 = el->vtab->removeChild(el, node4, &exc);
	el->vtab->insertBefore(el, node4, node3, &exc);
	el->vtab->insertBefore(el, node5, node4, &exc);

  /* Add indentation */
	tnode = nl->vtab->item(nl, 0L, &exc);
  cnode = tnode->vtab->cloneNode(tnode, 0, &exc);
  el->vtab->insertBefore(el, cnode, node5, &exc);
  cnode = tnode->vtab->cloneNode(tnode, 0, &exc);
  el->vtab->insertBefore(el, cnode, node4, &exc);
  cnode = tnode->vtab->cloneNode(tnode, 0, &exc);
  el->vtab->insertBefore(el, cnode, node3, &exc);
  cnode = tnode->vtab->cloneNode(tnode, 0, &exc);
  el->vtab->insertBefore(el, cnode, node2, &exc);
  cnode = tnode->vtab->cloneNode(tnode, 0, &exc);
  el->vtab->insertBefore(el, cnode, node1, &exc);
	tnode->vtab->set_nodeValue(tnode,gdome_xml_str_mkref("\n"),&exc);
	el->vtab->appendChild(el, tnode, &exc);

	/* delete old indentation */
	/* if nodelist is not live this doesn't work fine */
	for (i=0; i<5; i++) {
		tnode = nl->vtab->item(nl, 0L, &exc);
		el->vtab->removeChild(el, tnode, &exc);
	}
  dot();

	fprintf(stderr,"test_node\n");

  return ret;
}

gboolean
test_df_node(GdomeNode *el) {
	GdomeDocumentFragment *df;
	GdomeDocument *doc;
	GdomeElement *el1, *el2, *el3, *tel, *root;
	GdomeComment *cmt1, *cmt2;
  GdomeText *txt1, *txt2, *txt3, *txt4, *txt5, *txt6;
	GdomeAttr *attr1,*attr2;
  GdomeNodeList *nl;
  GdomeException exc;

	doc = el->vtab->ownerDocument (el, &exc);
	df = doc->vtab->createDocumentFragment (doc, &exc);
  dot();

	/* Creation of comment nodes */
  cmt1 = doc->vtab->createComment(doc, gdome_xml_str_mkref("*** Start of the new nodes ***"), &exc);
	cmt2 = doc->vtab->createComment(doc, gdome_xml_str_mkref("*** Stop of the new nodes ***"), &exc);
  dot();

	/* Creation of attribute nodes */
  attr1 = doc->vtab->createAttribute(doc, gdome_xml_str_mkref("DTATTR1"), &exc);
  attr1->vtab->set_value(attr1, gdome_xml_str_mkref("dtattr1"), &exc);
	attr2 = doc->vtab->createAttribute(doc, gdome_xml_str_mkref("DTATTR2"), &exc);
	attr2->vtab->set_value(attr2, gdome_xml_str_mkref("dtattr2"), &exc);
  dot();

	/* Creation of text nodes for indentation */
  txt1 = doc->vtab->createTextNode(doc, gdome_xml_str_mkref("\n  "), &exc);
  txt2 = doc->vtab->createTextNode(doc, gdome_xml_str_mkref("\n  "), &exc);
  txt3 = doc->vtab->createTextNode(doc, gdome_xml_str_mkref("\n  "), &exc);
  txt4 = doc->vtab->createTextNode(doc, gdome_xml_str_mkref("\n  "), &exc);
  txt5 = doc->vtab->createTextNode(doc, gdome_xml_str_mkref("\n  "), &exc);
  txt6 = doc->vtab->createTextNode(doc, gdome_xml_str_mkref("\n    "), &exc);
  dot();

	/* Creation of element nodes */
	el1 = doc->vtab->createElement(doc, gdome_xml_str_mkref("DTEL1"), &exc);
  el1->vtab->setAttributeNode(el1, attr1, &exc);
	el2 = doc->vtab->createElement(doc, gdome_xml_str_mkref("DTEL2"), &exc);
	el2->vtab->setAttributeNode(el2, attr2, &exc);
  el1->vtab->super.appendChild((GdomeNode *)el1, (GdomeNode *)txt6, &exc);
  el1->vtab->super.appendChild((GdomeNode *)el1, (GdomeNode *)el2, &exc);
  el1->vtab->super.appendChild((GdomeNode *)el1, (GdomeNode *)txt5, &exc);
	el3 = doc->vtab->createElement(doc, gdome_xml_str_mkref("DTEL3"), &exc);
  dot();

	/* Add theese Nodes to the Document Fragment */
  df->vtab->super.appendChild((GdomeNode *)df, (GdomeNode *)cmt1, &exc);
  df->vtab->super.appendChild((GdomeNode *)df, (GdomeNode *)txt1, &exc);
  df->vtab->super.appendChild((GdomeNode *)df, (GdomeNode *)el1, &exc);
  df->vtab->super.appendChild((GdomeNode *)df, (GdomeNode *)txt2, &exc);
  df->vtab->super.appendChild((GdomeNode *)df, (GdomeNode *)el3, &exc);
  df->vtab->super.appendChild((GdomeNode *)df, (GdomeNode *)txt3, &exc);
	df->vtab->super.appendChild((GdomeNode *)df, (GdomeNode *)cmt2, &exc);
  df->vtab->super.appendChild((GdomeNode *)df, (GdomeNode *)txt4, &exc);
  dot();

	nl = doc->vtab->getElementsByTagName(doc, gdome_xml_str_mkref("NODE3"), &exc);
  tel = (GdomeElement *)nl->vtab->item(nl, 0L, &exc);
	root = (GdomeElement *)tel->vtab->super.parentNode((GdomeNode *)tel, &exc);
	root->vtab->super.insertBefore((GdomeNode *)root, (GdomeNode *)df, (GdomeNode *)tel, &exc);
  dot();

	fprintf(stderr,"test_df_node\n");

  return TRUE;
}

int
main (int argc, char **argv)
{
	GdomeDOMImplementation *domImpl = NULL;
  GdomeDocument *domdoc = NULL;
  GdomeElement *el;
	GdomeException exc = 0;
	gboolean ret;

	domImpl = gdome_DOMImplementation_mkref();
	domdoc = domImpl->vtab->parseFile(domImpl, LOCALDIR"/test-node.xml", &exc);
  if (domdoc == NULL) {
      fprintf (stderr, "DOImplementation.parseFile: failed\n   test-node.xml not found\n");
      return -1;
  }
  el = domdoc->vtab->documentElement (domdoc, &exc);
	ret = test_node((GdomeNode *)el);
	ret = test_df_node((GdomeNode *)el);
	domImpl->vtab->saveFile(domImpl, "output.xml", domdoc, &exc);
  gdome_n_unref ((GdomeNode *)el, &exc);
  domImpl->vtab->freeDoc(domImpl, domdoc, &exc);

	if(ret)
		return 0;
	else
		return -1;
}
